{% import 'enumeration.jinja' as enumerate %}
# Pipeline overview


![**dag**](dag.png)


**Figure 1. Directed Acyclic Graph (DAG) containing a representation of the flow of the steps executed, as generated by
[Snakemake](https://snakemake.readthedocs.io/en/stable/tutorial/basics.html#step-4-indexing-read-alignments-and-visualizing-the-dag-of-jobs).**


{{ enumerate.enumerate_pipeline_steps(steps) }}