# useful fs constants
MF_FP = "/batchx/manifest/manifest.json"
INPUT_FP = "/batchx/input/input.json"
OUTPUT_FP = "/batchx/output/output.json"
DAG_FP = "/batchx/output/dag.svg"
SNAKEMAKE_EXIT_CODE_FP = "/tmp/snakemake_exit_code.json"

# runtime constants
DEFAULT_MEMORY_MB = 4096
DEFAULT_VCPUS = 1
